@@ -41,46 +41,83 @@
 extern void sock_init(void);
 extern void do_initcalls(void);
 extern void init_timervecs (void);
-extern void timer_bh(void);
+extern void run_local_timers(void);
 extern void softirq_init(void);
 
-struct task_struct *nsc_task;
 extern struct proc_dir_entry *proc_net, *proc_sys_root;
+extern int nsc_g_l26_diagnostic;
+
+void nsc_sysctl_a(void);
+struct thread_info *nsc_get_current_thread_info(void);
+
+struct task_struct *nsc_task;
 
 void nsc_init(void)
 {
+    extern int nsc_sock_init_inodecache(void);
+
     nsc_task = (struct task_struct *)nsc_malloc(sizeof(struct task_struct));
     memset(nsc_task, 0, sizeof(*nsc_task));
 
     current->cap_effective = 0xFFFFFFFF;
     current->euid = 0;
-    init_timervecs(); // sched_init() calls this
+    current->thread_info = (struct thread_info *)nsc_malloc(sizeof(struct thread_info));
+    memset(current->thread_info, 0, sizeof(struct thread_info));
+    current->thread_info->addr_limit.seg = 0xFFFFFFFF;
+    
+    // XXX: 2.6:
+    // In 2.4, sched_init() calls this. Not sure about 2.6 yet...
+    //init_timervecs(); 
+    init_timers();
     softirq_init();
 
+    kmem_cache_init();
+
     proc_net = proc_mkdir("net", 0);
     proc_sys_root = proc_mkdir("sys", 0);
     
     sysctl_init();
+
+    nsc_sock_init_inodecache();
+    
     sock_init();
 
     do_initcalls();
 
     // hrm. Hax for now:
-    init_task.next_task = &init_task;
+    // // NOTE! This does not work in 2.6! Need to figure out why
+    // this was here in the first place! Jan 5, 2005
+    //  init_task.next_task = &init_task;
+}
+
+
+struct thread_info *nsc_get_current_thread_info()
+{
+    return nsc_task->thread_info;
 }
 
 extern volatile unsigned long jiffies;
 
 void nsc_timer_tick(void)
 {
-    nsc_debugf("nsc_timer_tick\n");
-    do_timer(NULL);
+    extern volatile unsigned long jiffies;
+    struct pt_regs regs;
+    memset(&regs, 0, sizeof(regs));
+    //nsc_debugf("nsc_timer_tick\n");
+    jiffies += 1;
+    do_timer(&regs);
 }
 
 void nsc_timer_interrupt(void)
 {
-    nsc_debugf("nsc_timer_interrupt\n");
-    timer_bh();
+    extern asmlinkage void do_softirq(void);
+
+    //nsc_debugf("nsc_timer_interrupt\n");
+    // XXX: 2.4 uses timer_bh:
+    //timer_bh();
+    run_local_timers();
+    // solution on 2.6:
+    do_softirq();
 }
 
 static void nsc_if_set_addr(unsigned int ip_addr, unsigned int ip_mask)
@@ -133,9 +170,21 @@ void nsc_if_receive_packet(void *dev, void *data, unsigned int datalen)
 {
     struct sk_buff *skb;
     // To be consistent with the eepro100 driver.
-    unsigned int skbuff_size = 1536;
+    const unsigned int skbuff_size = 1536;
 
-    if(datalen <= 1536) {
+    // Bit of a hack...
+    // Note that the size allocated here effects the offered window somewhat.
+    // I've got this heuristic here to try and match up with what we observe
+    // on the emulation network and by looking at the driver code of the
+    // eepro100. In both cases we allocate enough space for our packet, which
+    // is the important thing. The amount of slack at the end can make linux
+    // decide the grow the window differently. This is quite subtle, but the
+    // code in question is in the tcp_grow_window function. It checks
+    // skb->truesize, which is the size of the skbuff allocated for the 
+    // incoming data packet -- what we are allocating right now!
+    if(datalen < 1200) {
+        skb = dev_alloc_skb(datalen + 36);
+    } else if(datalen <= 1500) {
         skb = dev_alloc_skb(skbuff_size);
     } else {
         skb = dev_alloc_skb(datalen + 36);
@@ -169,14 +218,24 @@ static int nsc_if_xmit(struct sk_buff *skb, struct net_device *dev)
     return 0;
 }
 
+void nsc_netdev_setup(struct net_device *a)
+{
+    // Nothing
+}
+
 void *nsc_newif(unsigned int addr, unsigned int mask, int mtu)
 {
     struct net_device *ifp;
     int error;
 
-    ifp = dev_alloc("sa%d", &error);
-
-    nsc_assert(error == 0, "dev_alloc failed");
+    // 2.4 style:
+    //ifp = dev_alloc("sa%d", &error);
+    // New for 2.6:
+    ifp = alloc_netdev(0, "sa%d", &nsc_netdev_setup);
+    nsc_assert(ifp != 0, "alloc_netdev failed");
+    
+    error = dev_alloc_name(ifp, ifp->name);
+    nsc_assert(error == 0, "dev_alloc_name failed");
 
     ifp->mtu = mtu; // XXX: Check this works
     //ifp->features = NETIF_F_HW_CSUM;
@@ -231,7 +290,7 @@ void nsc_soconnect(void *so, unsigned int dest, unsigned short port)
     addr.sin_addr.s_addr = dest;
     
     addrlen = sizeof(struct sockaddr_in);
-    error = sock->ops->connect(sock, (struct sockaddr *)&addr, addrlen, 0);
+    error = sock->ops->connect(sock, (struct sockaddr *)&addr, addrlen, O_NONBLOCK);
 
     // XXX: haxored haxored haxored, this is bad
 //    if(sock->proto == IPPROTO_SCTP)
@@ -282,7 +341,7 @@ int nsc_accept(void *so, void **a)
     // the following function. We kinda don't support files... The last
     // argument just means the accepted socket inherits flags from so. We
     // don't really care about this.
-    error = sock->ops->accept(sock, newsock, 0);
+    error = sock->ops->accept(sock, newsock, O_NONBLOCK);
     
     if(error < 0) {
         sock_release(newsock);
@@ -337,7 +396,7 @@ int nsc_sosend(void *so, void *data, unsigned int datalen)
     msg.msg_controllen = 0;
     msg.msg_iov = (struct iovec *)&iov;
     msg.msg_iovlen = 1;
-    msg.msg_flags = 0; //MSG_DONTWAIT;// : 0;
+    msg.msg_flags = MSG_DONTWAIT;// : 0;
 
     if (sock->type == SOCK_SEQPACKET)
         msg.msg_flags |= MSG_EOR;
@@ -369,7 +428,7 @@ int nsc_soreceive(void *so, void *buf, int *buflen)
     msg.msg_name = address;
     msg.msg_namelen = MAX_SOCK_ADDR;
 
-    flags = 0; //MSG_DONTWAIT;
+    flags = MSG_DONTWAIT;
     
     error = sock_recvmsg(sock, &msg, *buflen, flags);
     if(error > 0) {
@@ -391,12 +450,13 @@ int nsc_is_connected(void *so)
      * are in the SS_CONNECTING state to progress to the SS_CONNECTED state.
      * The parameters are ignored because we have already initiated the
      * connection attempt. 
-     * - Sam, Feb 22 2005. */
+     * - Sam, Feb 22 2005. (copied from linux-2.4) */
     if(sock->sk && sock->state == SS_CONNECTING) {
         nsc_soconnect(so, 0, 0);
     }
 
     return sock->sk && sock->state == SS_CONNECTED;
+
 }
 
 // ------------------------------------------------------------------------
@@ -672,6 +732,7 @@ int nsc_do_sysctl(const char *sysctl_name, void *oldval, size_t *oldlenp,
             if(nlen != 3) return 1;
             break;
         }
+
     } while( (s = strchr(s, '/')) );
 
     if(s == NULL)
@@ -687,7 +748,6 @@ int nsc_setsockopt(void *so, char *optname, void *val, size_t valsize)
 
     if( strcmp(optname, "SO_SNDBUF") == 0 ) opt = SO_SNDBUF;
     else if( strcmp(optname, "SO_RCVBUF") == 0 ) opt = SO_RCVBUF;
-    else if( strcmp(optname, "TCP_QUICKACK") == 0 ) opt = TCP_QUICKACK;
     else return 1;
     
     return sock_setsockopt(sock, 0, opt, (char *)val, valsize);
@@ -696,7 +756,7 @@ int nsc_setsockopt(void *so, char *optname, void *val, size_t valsize)
 int nsc_get_tcp_var(void *so, const char *var, char *result, int rlen)
 {
     struct sock *sock = ((struct socket *)so)->sk;
-    struct tcp_opt *tp = &sock->tp_pinfo.af_tcp;
+    struct tcp_opt *tp = tcp_sk(sock);
 
          if(strcmp(var, "srtt_") == 0)  // in ticks
     {
@@ -741,7 +801,6 @@ int nsc_get_tcp_var(void *so, const char *var, char *result, int rlen)
     return 0;
 }
 
-void nsc_sysctl_a(void);
 void nsc_sysctl_r(struct proc_dir_entry *, const char *);
 
 void nsc_sysctl_a()
@@ -771,14 +830,16 @@ void nsc_sysctl_r(struct proc_dir_entry *p, const char *d)
         if(table->proc_handler)
         {
             char rbuf[6000];
-            unsigned int len = 5000;
+            size_t len = 5000;
+            loff_t pos = 0;
             struct file f;
             f.f_pos = 0;
 
             // Easiest way to null terminate the string:
             memset(rbuf, 0, 5000);
             
-            table->proc_handler(table, 0, &f, rbuf, &len);
+            // TODO: make this work for 2.6, below is for 2.4
+            table->proc_handler(table, 0, &f, rbuf, &len, &pos);
 
             if(strlen(rbuf)) {
                 printk("%s = ", buf2);
