@@ -19,7 +19,7 @@
   Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
 
-  $Id: nsc.cc 1255 2006-04-10 08:38:28Z stj2 $
+  $Id: nsc.cc 1294 2006-04-20 15:25:01Z stj2 $
   
   */
 
@@ -199,7 +199,13 @@ void NSCAgent::sendmsg(int nbytes, AppData* data, const char* flags)
         fprintf(stderr, "Attempt to send data when not connected!\n");
 #endif
 
-        precon_sndbuf_.push_back(nbytes);
+        // Don't let it grow to large for a TCP flow that never gets around
+        // to connecting.
+        if(precon_sndbuf_.size() > 20) {
+            precon_sndbuf_.back() += nbytes;
+        } else {
+            precon_sndbuf_.push_back(nbytes);
+        }
         return;
     }
 
@@ -1035,9 +1041,9 @@ static class Linux26SimpleAgentClass : public TclClass {
         }
 } class_linux26_simple_agent;
 
-static class Linux26SimpleAgentClass : public TclClass {
+static class Linux26142SimpleAgentClass : public TclClass {
     public:
-        Linux26SimpleAgentClass() : TclClass("Agent/TCP/NSC/Linux26142") {}
+        Linux26142SimpleAgentClass() : TclClass("Agent/TCP/NSC/Linux26142") {}
         TclObject* create(int, const char*const*) {
             return (new NSCSimpleAgent("liblinux2.6.14.2.so"));
         }
